searchData={"content_type":"text/markdown","items":[{"doc":"Defines a repository for Tablestore.\n\nA repository maps to an underlying data store, controlled by Ecto.Adapters.Tablestore adapter.\n\nWhen used, the repository expects the `:otp_app` option, and uses Ecto.Adapters.Tablestore by\ndefault.  The `:otp_app` should point to an OTP application that has repository configuration.\nFor example, the repository:\n\n```elixir\ndefmodule EctoTablestore.MyRepo do\n  use EctoTablestore.Repo,\n    otp_app: :my_otp_app\nend\n```\n\nConfigure `ex_aliyun_ots` as usual:\n\n```elixir\nconfig :ex_aliyun_ots, MyInstance,\n  name: \"MY_INSTANCE_NAME\",\n  endpoint: \"MY_INSTANCE_ENDPOINT\",\n  access_key_id: \"MY_OTS_ACCESS_KEY\",\n  access_key_secret: \"MY_OTS_ACCESS_KEY_SECRET\"\n\nconfig :ex_aliyun_ots,\n  instances: [MyInstance]\n```\n\nAdd the following configuration to associate `MyRepo` with the previous configuration of\n`ex_aliyun_ots`:\n\n```elixir\nconfig :my_otp_app, EctoTablestore.MyRepo,\n  instance: MyInstance\n```","ref":"EctoTablestore.Repo.html","title":"EctoTablestore.Repo","type":"behaviour"},{"doc":"Returns the adapter tied to the repository.","ref":"EctoTablestore.Repo.html#c:__adapter__/0","title":"EctoTablestore.Repo.__adapter__/0","type":"callback"},{"doc":"Batch get several rows of data from one or more tables, this batch request put multiple\n`get_row` in one request from client's perspective.\n\nAfter execute each operation in servers, return results independently and independently consumes\ncapacity units.\n\nWhen input `schema_entity`, only theirs primary keys are used in query, if need to use theirs\nattribute columns into condition of query, please use `entity_full_match: true` option to do\nthat.","ref":"EctoTablestore.Repo.html#c:batch_get/1","title":"EctoTablestore.Repo.batch_get/1","type":"callback"},{"doc":"When the schema has only 1 primary key, and you use default options:\n\n    # get_row style\n    batch_get([\n      {Schema1, [[{\"id\", 1}], [{\"id\", 2}]]},\n      {Schema2, [[{\"id\", 3}], [{\"id\", 4}]]}\n    ])\n\n    # get_row style\n    # if only get one row from one schema could also be:\n    batch_get([\n      {Schema1, [{\"id\", 1}]},\n      {Schema2, [{\"id\", 3}]}\n    ])\n\n    # schema_entity style\n    batch_get([\n      [%Schema1{id: 1}, %Schema1{id: 2}],\n      [%Schema2{id: 3}, %Schema2{id: 4}]\n    ])\n\n\nWhen the schema has only 1 primary key, and you use custom options:\n\n    # get_row style\n    batch_get([\n      {Schema1, [{\"id\", 1}], [{\"id\", 2}], columns_to_get: [\"field1\"]},\n      {Schema2, [{\"id\", 3}], [{\"id\", 4}], columns_to_get: [\"field2\"]}\n    ])\n\n    # schema_entity style\n    batch_get([\n      {\n        [%Schema1{id: 1}, %Schema1{id: 2}],\n        columns_to_get: [\"field1\"]\n      },\n      {\n        [%Schema2{id: 3}, %Schema2{id: 4}],\n        columns_to_get: [\"field2\"]\n      }\n    ])\n\nEntity full match:\n\n    # only schema_entity style\n    batch_get([\n      {\n        [%Schema1{id: 1, field1: 10}, %Schema1{id: 2, field1: 20}],\n        entity_full_match: true\n      },\n      {\n        [%Schema2{id: 3, field2: 30}, %Schema2{id: 4, field2: 40}],\n        entity_full_match: true\n      }\n    ])\n\nWhen the schema has multiple primary keys:\n\n    # get_row style\n    batch_get([\n      {Schema3, [\n        [{\"id\", 1}, {\"another_pk\", \"example1\"}],\n        [{\"id\", 2}, {\"another_pk\", \"example2\"}]\n      ]},\n      {Schema4, [\n        [{\"id\", 3}, {\"another_pk\", \"example3\"}],\n        [{\"id\", 4}, {\"another_pk\", \"example4\"}]\n      ]}\n    ])\n\n    # schema_entity style\n    batch_get([\n      [\n        %Schema3{id: 1, another_pk: \"example1\"},\n        %Schema3{id: 2, another_pk: \"example2\"}\n      ],\n      [\n        %Schema4{id: 3, another_pk: \"example3\"},\n        %Schema4{id: 4, another_pk: \"example4\"}\n      ]\n    ])","ref":"EctoTablestore.Repo.html#c:batch_get/1-example","title":"Example - EctoTablestore.Repo.batch_get/1","type":"callback"},{"doc":"Batch write several rows of data from one or more tables, this batch request puts multiple\nPutRow/DeleteRow/UpdateRow operations in one request from client's perspective.\n\nAfter execute each operation in servers, return results independently and independently consumes\ncapacity units.\n\nIf use a batch write request include a transaction ID, all rows in that request can only be\nwritten to the table that matches the transaction ID.","ref":"EctoTablestore.Repo.html#c:batch_write/2","title":"EctoTablestore.Repo.batch_write/2","type":"callback"},{"doc":"* `transaction_id`, use local transaction.","ref":"EctoTablestore.Repo.html#c:batch_write/2-options","title":"Options - EctoTablestore.Repo.batch_write/2","type":"callback"},{"doc":"The options of each `:put`, `:delete`, and `:update` operation are similar as\n`ExAliyunOts.put_row/5`, `ExAliyunOts.delete_row/4` and `ExAliyunOts.update_row/4`, but\n`transaction_id` option is using in the options of `c:EctoTablestore.Repo.batch_write/2`.\n\nBy default, require to explicitly set the `:condition` option in each operation, excepts that\nif a table defined an auto increment primary key(aka non-partitioned primary key) which is processed in the server side,\nthe server logic MUST use `condition: condition(:ignore)`, in this case, this library internally forces to use\n`condition: condition(:ignore)`, so we can omit the `:condition` option in the PutRow operation of a batch write.\n\nIf we set `entity_full_match: true` there will use the whole provided attribute-column field(s) of\nschema entity into the `column_condition` of condition filter, and always use `row_existence: :EXPECT_EXIST`,\nby default the `entity_full_match` option is `false`.\n\nIf put a row with an auto increment primary key, meanwhile set `entity_full_match: true`, the `entity_full_match: true`\noption is no effect, this library internally forces to use `condition: condition(:ignore)`.\n\n    batch_write([\n      delete: [\n        {schema_entity_1, condition: condition(:ignore)}\n        {schema_entity_2, condition: condition(:expect_exist)}\n      ],\n      put: [\n        {%Schema1{}, condition: condition(:expect_not_exist)},\n        {%Schema2{}, condition: condition(:ignore)},\n        {%Schema3WithAutoIncrementPK{}, return_type: :pk},\n        {changeset_schema_1, condition: condition(:ignore)}\n      ],\n      update: [\n        {changeset_schema_1, return_type: :pk},\n        {changeset_schema_2, entity_full_match: true}\n      ]\n    ])\n\nUse `transaction_id` option:\n\n    batch_write(\n      [\n        delete: [...],\n        put: [...],\n        update: [...]\n      ],\n      transaction_id: \"...\"\n    )","ref":"EctoTablestore.Repo.html#c:batch_write/2-example","title":"Example - EctoTablestore.Repo.batch_write/2","type":"callback"},{"doc":"Delete a struct using its primary key.","ref":"EctoTablestore.Repo.html#c:delete/2","title":"EctoTablestore.Repo.delete/2","type":"callback"},{"doc":"* `:condition`, this option is required, whether to add conditional judgment before data\n    delete.\n\n      1. As `condition(:expect_exist)` means the primary key(s) can match a row to delete, if the row is existed, \n         the delete result will be success, if the row is not existed, the delete result will be fail.\n\n           We also can add some compare expressions for the attribute columns, e.g:\n\n             condition(:expect_exist, \"attr1\" == value1 and \"attr2\" > 1)\n             condition(:expect_exist, \"attr1\" != value1)\n             condition(:expect_exist, \"attr1\" > 100 or \"attr2\" < 1000)\n\n      2. As `condition(:ignore)` means DO NOT do any condition validation before delete, whether the row exists or not,\n         both of the delete results will be success.\n\n      3. As `condition(:expect_not_exist)` means expect the primary key(s) are NOT existed before delete, if the row\n         is existed, the delete result will be fail, if the row is not existed, the delete result will be success.\n\n  * `:transaction_id`, delete under local transaction in a partition key.\n  * `:stale_error_field` - The field where stale errors will be added in the returning\n    changeset.  This option can be used to avoid raising `Ecto.StaleEntryError`.\n  * `:stale_error_message` - The message to add to the configured `:stale_error_field` when\n    stale errors happen, defaults to \"is stale\".","ref":"EctoTablestore.Repo.html#c:delete/2-options","title":"Options - EctoTablestore.Repo.delete/2","type":"callback"},{"doc":"Fetch a single struct from tablestore where the whole primary key(s) match the given ids.","ref":"EctoTablestore.Repo.html#c:get/3","title":"EctoTablestore.Repo.get/3","type":"callback"},{"doc":"* `:columns_to_get`, string list, return the specified attribute columns, if not specify this\n  option field, will try to return all attribute columns together.\n* `:start_column`, string, used as a starting column for Wide Column read, the return result\n  contains this as starter.\n* `:end_column`, string, used as a ending column for Wide Column read, the return result DON NOT\n  contain this column.\n* `:filter`, used as a filter by condition, support `\">\"`, `\"<\"`, `\">=\"`, `\"<=\"`, `\"==\"`,\n  `\"and\"`, `\"or\"` and `\"()\"` expressions.\n\n    The `ignore_if_missing` option can be used for the non-existed attribute column, for\n    example:\n\n    An attribute column does not exist meanwhile set it as `true`, will ignore this match\n    condition in the return result;\n\n    An existed attribute column DOES NOT suit for this use case, the match condition will always\n    affect the return result, if match condition does not satisfy, they won't be return in\n    result.\n\n    ```elixir\n    filter: filter(({\"name\", ignore_if_missing: true} == var_name and \"age\" > 1) or (\"class\" == \"1\"))\n    ```\n* `:transaction_id`, read under local transaction in a partition key.","ref":"EctoTablestore.Repo.html#c:get/3-options","title":"Options - EctoTablestore.Repo.get/3","type":"callback"},{"doc":"Get multiple structs by range to the schema, result in `:forward` direction by default.","ref":"EctoTablestore.Repo.html#c:get_range/2","title":"EctoTablestore.Repo.get_range/2","type":"callback"},{"doc":"Simply get result in a range request:\n\n    Repo.get_range(Order)\n    Repo.get_range(Order, direction: :backward)\n\n  If the `Order` schema has multiple primary keys, and the `:id` field is the partition key,\n  we can get records in range under the partition key `\"1\"` in this way:\n\n    Repo.get_range(%Order{id: \"1\"})","ref":"EctoTablestore.Repo.html#c:get_range/2-example","title":"Example - EctoTablestore.Repo.get_range/2","type":"callback"},{"doc":"See `c:get_range/4`.","ref":"EctoTablestore.Repo.html#c:get_range/2-options","title":"Options - EctoTablestore.Repo.get_range/2","type":"callback"},{"doc":"Get multiple structs by range from one table, rely on the conjunction of the partition key and\nother primary key(s).","ref":"EctoTablestore.Repo.html#c:get_range/4","title":"EctoTablestore.Repo.get_range/4","type":"callback"},{"doc":"* `:direction`, by default it is `:forward`, set it as `:forward` to make the order of the query\n  result in ascending by primary key(s), set it as `:backward` to make the order of the query\n  result in descending by primary key(s).\n* `:columns_to_get`, string list, return the specified attribute columns, if not specify this\n  field all attribute columns will be return.\n* `:start_column`, string, used as a starting column for Wide Column read, the return result\n  contains this as starter.\n* `:end_column`, string, used as a ending column for Wide Column read, the return result DON NOT\n  contain this column.\n* `:limit`, optional, the maximum number of rows of data to be returned, this value must be\n  greater than 0, whether this option is set or not, there returns a maximum of 5,000 data rows\n  and the total data size never exceeds 4 MB.\n* `:transaction_id`, read under local transaction in a partition key.\n* `:filter`, used as a filter by condition, support `\">\"`, `\"<\"`, `\">=\"`, `\"<=\"`, `\"==\"`,\n  `\"and\"`, `\"or\"` and `\"()\"` expressions.\n\n    The `ignore_if_missing` option can be used for the non-existed attribute column, for\n    example:\n\n    An attribute column does not exist meanwhile set it as `true`, will ignore this match\n    condition in the return result;\n\n    An existed attribute column DOES NOT suit for this use case, the match condition will always\n    affect the return result, if match condition does not satisfy, they won't be return in\n    result.\n\n    ```elixir\n    filter: filter(({\"name\", ignore_if_missing: true} == var_name and \"age\" > 1) or (\"class\" == \"1\"))\n    ```","ref":"EctoTablestore.Repo.html#c:get_range/4-options","title":"Options - EctoTablestore.Repo.get_range/4","type":"callback"},{"doc":"Returns true if the current process is inside a transaction.","ref":"EctoTablestore.Repo.html#c:in_transaction?/0","title":"EctoTablestore.Repo.in_transaction?/0","type":"callback"},{"doc":"MyRepo.in_transaction?\n    #=> false\n\n    MyRepo.transaction(fn ->\n      MyRepo.in_transaction? #=> true\n    end)","ref":"EctoTablestore.Repo.html#c:in_transaction?/0-examples","title":"Examples - EctoTablestore.Repo.in_transaction?/0","type":"callback"},{"doc":"Inserts a struct defined via EctoTablestore.Schema or a changeset.\n\nIf a table defined an auto increment primary key which is processed in the server side, the server logic\nMUST use `condition(:ignore)`, in this case, this library internally forces to use `condition(:ignore)` to `:condition`\noption, so here we can omit the `:condition` option when insert a row.","ref":"EctoTablestore.Repo.html#c:insert/2","title":"EctoTablestore.Repo.insert/2","type":"callback"},{"doc":"* `:condition`, this option is required excepts that the table defined an auto increment primary key,\n    whether to add conditional judgment before date insert.\n\n    1. As `condition(:ignore)` means DO NOT do any condition validation before insert, whether the row exists or not,\n       both of the insert results will be success, if the schema non-partitioned primary key is auto increment,\n       we can only use `condition(:ignore)` option.\n    2. As `condition(:expect_not_exist)` means expect the primary key(s) are NOT existed before insert, if the row is\n       existed, the insert result will be fail, if the row is not existed, the insert result will be success.\n    3. As `condition(:expect_exist)` means overwrite the whole existed row to the primary key(s), if the row is existed,\n       the insert result will be success, if the row is not existed, the insert result will be fail.\n\n  * `:transaction_id`, insert under local transaction in a partition key.","ref":"EctoTablestore.Repo.html#c:insert/2-options","title":"Options - EctoTablestore.Repo.insert/2","type":"callback"},{"doc":"Similar to `c:get/3`, please ensure schema entity has been filled with the whole primary key(s).","ref":"EctoTablestore.Repo.html#c:one/2","title":"EctoTablestore.Repo.one/2","type":"callback"},{"doc":"* `:entity_full_match`, whether to transfer the input attribute column(s) into the `:==`\n    filtering expressions, by default it is `false`, when set `entity_full_match: true`, please\n    notice the following rules:\n\n     * If there exists attribute column(s) provided in entity, these fields will be combined\n      within multiple `:==` filtering expressions;\n\n     * If there exists attribute column(s) provided and meanwhile set `filter` option, they will\n     be merged into a composite filter.\n\nOther options please refer `c:get/3`.","ref":"EctoTablestore.Repo.html#c:one/2-options","title":"Options - EctoTablestore.Repo.one/2","type":"callback"},{"doc":"See `c:one/2` for more details.","ref":"EctoTablestore.Repo.html#c:one!/2","title":"EctoTablestore.Repo.one!/2","type":"callback"},{"doc":"Rolls back the current transaction.\n\nThe transaction will return the value given as `{:error, value}`.\n\nNote that calling `rollback` causes the code in the transaction to stop executing.\n\nSimilar to `c:Ecto.Repo.rollback/1`.","ref":"EctoTablestore.Repo.html#c:rollback/1","title":"EctoTablestore.Repo.rollback/1","type":"callback"},{"doc":"Provide search index features for the following scenarios:\n\n* MatchAllQuery\n* MatchQuery\n* MatchPhraseQuery\n* TermQuery\n* TermsQuery\n* PrefixQuery\n* RangeQuery\n* WildcardQuery\n* BoolQuery\n* NestedQuery\n* ExistsQuery\n* GeoBoundingBoxQuery\n* GeoDistanceQuery\n* GeoPolygonQuery\n\nPlease refer [ExAliyunOts.Search](https://hexdocs.pm/ex_aliyun_ots/ExAliyunOts.Search.html#query) query section\nfor details, they are similar options for use, for example:\n\n```\nMyRepo.search(MySchema, \"index_name\",\n  search_query: [\n    query: exists_query(\"comment\")\n  ]\n)\n```","ref":"EctoTablestore.Repo.html#c:search/3","title":"EctoTablestore.Repo.search/3","type":"callback"},{"doc":"Use SQL query data, only support select query.","ref":"EctoTablestore.Repo.html#c:sql_query/2","title":"EctoTablestore.Repo.sql_query/2","type":"callback"},{"doc":"sql_query(Schema1, \"SELECT * FROM table LIMIT 20\")","ref":"EctoTablestore.Repo.html#c:sql_query/2-example","title":"Example - EctoTablestore.Repo.sql_query/2","type":"callback"},{"doc":"Please see `c:Ecto.Repo.start_link/1` for details.","ref":"EctoTablestore.Repo.html#c:start_link/1","title":"EctoTablestore.Repo.start_link/1","type":"callback"},{"doc":"As a wrapper built on `stream_range/4` to create composable and lazy enumerables\nstream for iteration.","ref":"EctoTablestore.Repo.html#c:stream/2","title":"EctoTablestore.Repo.stream/2","type":"callback"},{"doc":"Please see options of `c:get_range/4` for details.","ref":"EctoTablestore.Repo.html#c:stream/2-options","title":"Options - EctoTablestore.Repo.stream/2","type":"callback"},{"doc":"As a wrapper built on `ExAliyunOts.stream_range/5` to create composable and lazy enumerables\nstream for iteration.","ref":"EctoTablestore.Repo.html#c:stream_range/4","title":"EctoTablestore.Repo.stream_range/4","type":"callback"},{"doc":"Please see options of `c:get_range/4` for details.","ref":"EctoTablestore.Repo.html#c:stream_range/4-options","title":"Options - EctoTablestore.Repo.stream_range/4","type":"callback"},{"doc":"As a wrapper built on `ExAliyunOts.stream_search/4` to create composable and lazy enumerables\nstream for iteration.","ref":"EctoTablestore.Repo.html#c:stream_search/3","title":"EctoTablestore.Repo.stream_search/3","type":"callback"},{"doc":"Please see options of `c:search/3` for details.","ref":"EctoTablestore.Repo.html#c:stream_search/3-options","title":"Options - EctoTablestore.Repo.stream_search/3","type":"callback"},{"doc":"Adapt to use the Tablestore's local transaction to perform read and write operations on the data, the feature can\nperform atomic operations to read and write one or more rows.","ref":"EctoTablestore.Repo.html#c:transaction/2","title":"EctoTablestore.Repo.transaction/2","type":"callback"},{"doc":"* `:table`, this option is required, specifies the table name to enable local transaction.\n  * `:partition_key`, this option is required as a tuple, specifies the partition key and its value to enable local transactions, for example `{\"partition_key_name\", \"partition_key_value\"}`","ref":"EctoTablestore.Repo.html#c:transaction/2-options","title":"Options - EctoTablestore.Repo.transaction/2","type":"callback"},{"doc":"```elixir\n{:ok, item_updated} = TestRepo.transaction(fn ->\n    item = TestRepo.one(item)\n\n    {:ok, updated} =\n      item\n      |> Changeset.change(array: [\"more\" | item.array])\n      |> TestRepo.update(\n        condition: :expect_exist,\n        stale_error_message: \"no_exist\",\n        stale_error_field: :condition,\n        returning: [:array]\n      )\n    updated\n  end,\n  table: \"TableName\", partition_key: {\"p_key\", \"pk_value1\"}\n)\n```","ref":"EctoTablestore.Repo.html#c:transaction/2-example","title":"Example - EctoTablestore.Repo.transaction/2","type":"callback"},{"doc":"Updates a changeset using its primary key.","ref":"EctoTablestore.Repo.html#c:update/2","title":"EctoTablestore.Repo.update/2","type":"callback"},{"doc":"* `:condition`, this option is required, whether to add conditional judgment before data update.\n\n    1. As `condition(:expect_exist)` means the primary key(s) can match a row to update, if the row is existed,\n       the update result will be success, if the row is not existed, the update result will be fail.\n\n         We also can add some compare expressions for the attribute columns, e.g:\n\n           condition(:expect_exist, \"attr1\" == value1 and \"attr2\" > 1)\n           condition(:expect_exist, \"attr1\" != value1)\n           condition(:expect_exist, \"attr1\" > 100 or \"attr2\" < 1000)\n\n    2. As `condition(:ignore)` means DO NOT do any condition validation before update, whether the row exists or not,\n       both of the update results will be success.\n\n    3. As `condition(:expect_not_exist)` means expect the primary key(s) are NOT existed before update, if the row\n       is existed, the update will be fail, if the row is not existed, the update result will be success.\n\n  * `:transaction_id`, update under local transaction in a partition key.\n  * `:stale_error_field` - The field where stale errors will be added in the returning\n    changeset. This option can be used to avoid raising `Ecto.StaleEntryError`.\n  * `:stale_error_message` - The message to add to the configured `:stale_error_field` when\n    stale errors happen, defaults to \"is stale\".\n  * `:returning`, this option is required when the input changeset with `:increment` operation, all fields of the atomic\n    increment operation are required to explicitly set into this option in any order, if missed any atomic increment\n    operation related field, there will raise an `Ecto.ConstraintError` to prompt and terminate this update.\n    If there is no `:increment` operation, the `:returning` option is no need to set. If set `returning: true`, but not\n    really all fields are changed, the unchanged fields will be replaced as `nil` in the returned schema data.","ref":"EctoTablestore.Repo.html#c:update/2-options","title":"Options - EctoTablestore.Repo.update/2","type":"callback"},{"doc":"","ref":"EctoTablestore.Repo.html#t:batch_gets/0","title":"EctoTablestore.Repo.batch_gets/0","type":"type"},{"doc":"","ref":"EctoTablestore.Repo.html#t:batch_writes/0","title":"EctoTablestore.Repo.batch_writes/0","type":"type"},{"doc":"","ref":"EctoTablestore.Repo.html#t:end_primary_keys/0","title":"EctoTablestore.Repo.end_primary_keys/0","type":"type"},{"doc":"","ref":"EctoTablestore.Repo.html#t:index_name/0","title":"EctoTablestore.Repo.index_name/0","type":"type"},{"doc":"","ref":"EctoTablestore.Repo.html#t:next_token/0","title":"EctoTablestore.Repo.next_token/0","type":"type"},{"doc":"","ref":"EctoTablestore.Repo.html#t:options/0","title":"EctoTablestore.Repo.options/0","type":"type"},{"doc":"","ref":"EctoTablestore.Repo.html#t:schema/0","title":"EctoTablestore.Repo.schema/0","type":"type"},{"doc":"","ref":"EctoTablestore.Repo.html#t:schema_or_changeset/0","title":"EctoTablestore.Repo.schema_or_changeset/0","type":"type"},{"doc":"","ref":"EctoTablestore.Repo.html#t:search_result/0","title":"EctoTablestore.Repo.search_result/0","type":"type"},{"doc":"","ref":"EctoTablestore.Repo.html#t:sql_query/0","title":"EctoTablestore.Repo.sql_query/0","type":"type"},{"doc":"","ref":"EctoTablestore.Repo.html#t:start_primary_keys/0","title":"EctoTablestore.Repo.start_primary_keys/0","type":"type"},{"doc":"","ref":"EctoTablestore.Repo.html#t:table_name/0","title":"EctoTablestore.Repo.table_name/0","type":"type"},{"doc":"","ref":"EctoTablestore.Repo.html#t:transaction_fun/0","title":"EctoTablestore.Repo.transaction_fun/0","type":"type"},{"doc":"","ref":"EctoTablestore.Repo.html#t:transaction_fun_abort_return/0","title":"EctoTablestore.Repo.transaction_fun_abort_return/0","type":"type"},{"doc":"","ref":"EctoTablestore.Repo.html#t:transaction_fun_commit_return/0","title":"EctoTablestore.Repo.transaction_fun_commit_return/0","type":"type"},{"doc":"","ref":"EctoTablestore.Repo.html#t:transaction_fun_return/0","title":"EctoTablestore.Repo.transaction_fun_return/0","type":"type"},{"doc":"Define an Ecto schema for Tablestore product use, there are some reasons make we do some wrapper base on `Ecto.Schema`:\n\n* Since Tablestore product does not support(also not recommend) the first primary key(aka the partition key) autogenerated\n  in server side, e.g. an auto increment sequence integer in server is not supported, so we set `@primary_key`\n  as `false` by default when use `EctoTablestore.Schema`.\n* Since Tablestore product's attribute column does not support DateTime or similar type in server side, so we use\n  `@timestamps_opts` to pre-configure `Ecto.Schema.timestamps/1` macro as an integer type for the generated `inserted_at`\n  and `updated_at` fields.\n\nIf you have explicitly set `@primary_key` and `@timestamps_opts`(or `Ecto.Schema.timestamps/1`) in the defined schema,\nand then you can write `use Ecto.Schema` to make the schema definition more compatible in Ecto use, for example:\n\n    defmodule User do\n      use Ecto.Schema\n\n      @primary_key {:id, :id, autogenerate: false}\n\n      schema \"user\" do\n        field(:name, :string)\n        timestamps(\n          type: :integer,\n          autogenerate: {System, :os_time, [:second]}\n        )\n      end\n    end\n\nIt equals to:\n\n    defmodule User do\n      use EctoTablestore.Schema\n\n      schema \"user\" do\n        field(:id, :id, primary_key: true)\n        field(:name, :string)\n        timestamps()\n      end\n    end\n\n#","ref":"EctoTablestore.Schema.html","title":"EctoTablestore.Schema","type":"module"},{"doc":"* The primary key supports `:id` (integer()) and `:binary_id` (binary()).\n* By default the `:primary_key` option is `false`.\n* The first defined primary key by the written order in the `schema` is the partition key.\n* Up to 4 primary key(s), it is limited by Tablestore product server side.\n* Up to 1 primary key with `autogenerate: true` option, it is limited by Tablestore product\n  server side.\n* The primary key set with `autogenerate: true` will use the Tablestore product server's\n  [AUTO_INCREMENT](https://www.alibabacloud.com/help/doc-detail/47745.htm) feature.\n* If the partition key set as `autogenerate: true` is not allowed to take advantage of the\n  AUTO_INCREMENT feature which it is limited by server, but there is a built-in implement to use\n  the `Sequence` to achieve the same atomic increment operation in `ecto_tablestore` library.\n\nHere is an example about the primary keys:\n\n    defmodule User do\n      use EctoTablestore.Schema\n\n      schema \"user\" do\n        field :outer_id, :binary_id, primary_key: true\n        field :internal_id, :id, primary_key: true, autogenerate: true\n        field :name, :string\n        field :desc\n      end\n    end\n\nBy default, if not explicitly set field type will process it as a `:string` type.","ref":"EctoTablestore.Schema.html#module-about-primary-key","title":"About primary key - EctoTablestore.Schema","type":"module"},{"doc":"Due to the previous implements, this macro is reserved to make compatible when `:hashids` or `EctoTablestore.Hashids`\ntype defined (changes in `0.13.0`).\n\nWe may deprecate it in the future, please use `Ecto.Schema.schema/2` instead.","ref":"EctoTablestore.Schema.html#tablestore_schema/2","title":"EctoTablestore.Schema.tablestore_schema/2","type":"macro"},{"doc":"Generate a migration to create a table of tablestore.\n\nThe repository must be set under `:ecto_repos` in the current app configuration,\nor given via the `-r` option:","ref":"Mix.Tasks.Ecto.Ots.Gen.Migration.html","title":"mix ecto.ots.gen.migration","type":"task"},{"doc":"1, Add configuration:\n\n    config :your_app, ecto_repos: [EctoTablestore.TestRepo]\n\n  2, Run:\n\n    mix ecto.ots.gen.migration create_posts","ref":"Mix.Tasks.Ecto.Ots.Gen.Migration.html#module-example-1","title":"Example 1 - mix ecto.ots.gen.migration","type":"task"},{"doc":"1, Run:\n\n    mix ecto.ots.gen.migration create_posts -r EctoTablestore.TestRepo\n\nThe generated migration filename will be prefixed with the current timestamp in UTC which is\nused for versioning and ordering.\n\nBy default, the migration will be generated to the \"priv/tablestore/YOUR_REPO/migrations\"\ndirectory of the current application but it can be configured to be any subdirectory of `priv`\nby specifying the `:priv` key under the repository configuration.","ref":"Mix.Tasks.Ecto.Ots.Gen.Migration.html#module-example-2","title":"Example 2 - mix ecto.ots.gen.migration","type":"task"},{"doc":"* `-r`, `--repo` - the repo to generate migration for","ref":"Mix.Tasks.Ecto.Ots.Gen.Migration.html#module-command-line-options","title":"Command line options - mix ecto.ots.gen.migration","type":"task"},{"doc":"Runs the all migration for the given repository.\n\nMigrations are expected at \"priv/tablestore/YOUR_REPO/migrations\" directory of the current\napplication, where \"YOUR_REPO\" is the last segment in your repository name. For example, the\nrepository `MyApp.Repo` will use \"priv/tablestore/repo/migrations\", the repository\n`Whatever.MyRepo` will use \"priv/tablestore/my_repo/migrations\".\n\nAlibaba Tablestore is a cloud service product, there will not do any local storage, meanwhile it\nis a NoSQL data storage, the migrations mainly responsible for the creation of the table primary\nkey(s).\n\nCurrently, run migrations will execute the all defined tasks, once Tablestore's primary key(s)\nare created, the primary keys(s) cannot be modified, failed to create an existing table, we can\nignore it.","ref":"Mix.Tasks.Ecto.Ots.Migrate.html","title":"mix ecto.ots.migrate","type":"task"},{"doc":"mix ecto.ots.migrate\n    mix ecto.ots.migrate -r EctoTablestore.TestRepo","ref":"Mix.Tasks.Ecto.Ots.Migrate.html#module-example","title":"Example - mix ecto.ots.migrate","type":"task"},{"doc":"* `-r`, `--repo` - the repo to migrate","ref":"Mix.Tasks.Ecto.Ots.Migrate.html#module-command-line-options","title":"Command line options - mix ecto.ots.migrate","type":"task"},{"doc":"A custom type leverages the generated sequence integer value to\nencode a hashid string for the primary key, recommend to use this for\nthe partition key (the first defined primary key).\n\nDefine `:hashids` type to the primary key of schema, for example:\n\n    defmodule MySchema do\n      use EctoTablestore.Schema\n\n      schema \"table_name\" do\n        field(:id, Ecto.Hashids, primary_key: true, autogenerate: true)\n      end\n\n    end\n\nIn the above case, there will try to find a schema `:hashids` configuration from\nthe application environment:\n\n    config :ecto_tablestore,\n      hashids: [\n        {MySchema, [salt: \"...\", min_len: ..., alphabet: \"...\"]}\n      ]\n\nIf the `:hashids` option of the application environment is not defined, there will use default\noptions(\"`[]`\") to `Hashids.new/1`.\n\nWe can also set the options when define the `:hashids` type to the primary key,\nfor example:\n\n    field(:id, :hashids,\n      primary_key: true,\n      autogenerate: true\n      hashids: [salt: \"...\", min_len: ..., alphabet: \"...\"]\n    )\n\nOnce the above `:hashids` options defined in the `:id` field of schema, there will always use them\nto encode a hashid, meanwhile the application runtime environment definition will not affect them.\n\nPlease see the options of `Hashids.new/1` for details.","ref":"Ecto.Hashids.html","title":"Ecto.Hashids","type":"module"},{"doc":"A custom type that replace some terms for strings.\n\n`Ecto.ReplaceableString` can be used when you want to replace some terms on dump/load\na field, or even both included, for example:\n\nBefore a field persisted to the database, `Ecto.ReplaceableString` replaces a matched pattern\nwith the replacement, the final replaced value will be saved into database.\n\n    field :content, Ecto.ReplaceableString,\n      on_dump: pattern: \"hello\", replacement: \"hi\", options: [global: false]\n\nAfter a field loaded from the database, `Ecto.ReplaceableString` replaces a matched pattern\nwith the replacement, the final replaced value will be extracted into the struct, but\nno changes into the original database.\n\n    field :content, Ecto.ReplaceableString,\n      on_load: pattern: ~r/test/, replacement: \"TEST\"\n\nThe `:pattern` and `:replacement` options are a pair required when both existed, the `:options` option\nis optional, these three options are correspond completely to `String.replace/4`.\n\nIf no `:on_dump` or `:on_load` option(s) set, the following cases are equal from result perspective, but\nrecommend to use the `:string` type as a base type:\n\n    field :content, Ecto.ReplaceableString\n    # equals\n    field :content, :string\n\nPlease notice that once used this type means a string replacement will be invoked in each read or write operation\nto the corresponding field, so please ensure use this type in a proper scenario to avoid a wasted performance issue,\nor find a better way to satisfy the similar use case.","ref":"Ecto.ReplaceableString.html","title":"Ecto.ReplaceableString","type":"module"},{"doc":"Migrations are used to create your tables.\n\nSupport the partition key is autoincrementing based on this library's wrapper, for this use\ncase, we can use the migration to automatically create an another separated table to generate\nthe serial value when `:insert` (viz `ExAliyunOts.put_row/5`) or `:batch_write` (viz\n`ExAliyunOts.batch_write/3`) with `:put` option.\n\nIn practice, we don't create migration files by hand either, we typically use `mix\necto.ots.gen.migration` to generate the file with the proper timestamp and then we just fill in\nits contents:\n\n    $ mix ecto.ots.gen.migration create_posts_table\n\nAnd then we can fill the table definition details:\n\n    defmodule EctoTablestore.TestRepo.Migrations.CreatePostsTable do\n      use EctoTablestore.Migration\n\n      def change do\n        create table(\"ecto_ots_test_posts\") do\n          add :post_id, :integer, partition_key: true, auto_increment: true\n        end\n      end\n    end\n\n\nAfter we filled the above migration content, you can run the migration above by going to the\nroot of your project and typing:\n\n    $ mix ecto.ots.migrate\n\nFinally, we successfully create the \"ecto_ots_test_posts\" table, since the above definition\nadded an autoincrementing column for the partition key, there will automatically create an\n\"ecto_ots_test_posts_seq\" table to generate a serial integer for `:post_id` field when insert a\nnew record.","ref":"EctoTablestore.Migration.html","title":"EctoTablestore.Migration","type":"module"},{"doc":"Adds a primary key when creating a table.\n\nThis function only accepts types as `:string` | `:binary` | `:integer` | `:hashids` | `:id`.\n\nAbout `:auto_increment` option:\n\n  * set `:auto_increment` as `true` and its field is primary key of non-partitioned key, there\n    will use Tablestore's auto-increment column to process it.\n\n  * set `:auto_increment` as `true` and its field is partition key, there will use\n    `ex_aliyun_ots`'s built-in Sequence function, the actual principle behind it is to use the\n    atomic update operation though another separate table when generate serial integer, by default\n    there will add an `:id` partition key as `:integer` type, the initial value of the sequence is\n    0, and the increment step is 1.\n\nTablestore can only have up to 4 primary keys, meanwhile the first defined primary key is the\npartition key, Please know that the order of the primary key definition will be directly mapped\nto the created table.\n\nAbout `:hashids` type to define the partition key:\n\n  * set `partition_key` as `true` is required.\n  * set `auto_increment` as `true` is required.","ref":"EctoTablestore.Migration.html#add/3","title":"EctoTablestore.Migration.add/3","type":"macro"},{"doc":"The auto generated serial integer for partition key:\n\n    create table(\"posts\") do\n      add :title, :string\n    end\n\n    # The above is equivalent to\n\n    create table(\"posts\", partition_key: false) do\n      add :id, :integer, partition_key: true, auto_increment: true\n      add :title, :string\n    end\n\nThe explicitly defined field with `partition_key`:\n\n    create table(\"posts\") do\n      add :title, :string\n    end\n\n    # The above is equivalent to\n\n    create table(\"posts\") do\n      add :id, :integer, partition_key: true, auto_increment: true\n      add :title, :string\n    end\n\nThe `:auto_increment` integer for primary key of non-partitioned key:\n\n    create table(\"posts\") do\n      add :tag, :integer, auto_increment: true\n    end\n\n    # The above is equivalent to\n\n    create table(\"posts\", partition_key: false) do\n      add :id, :integer, partition_key: true, auto_increment: true\n      add :version, :integer, auto_increment: true\n    end\n\nThe `:hashids` type for the partition key with the built-in sequence feature:\n\n    create table(\"posts\") do\n      add :id, :hashids, auto_increment: true, partition_key: true\n    end\n\nThe `:id` type for the partition key with the built-in sequence feature:\n\n    create table(\"posts\") do\n      add :id, :id\n    end\n\n    # The above is equivalent to\n\n    create table(\"posts\", partition_key: false) do\n      add :id, :integer, partition_key: true, auto_increment: true\n    end","ref":"EctoTablestore.Migration.html#add/3-examples","title":"Examples - EctoTablestore.Migration.add/3","type":"macro"},{"doc":"* `:partition_key` - when `true`, marks this field as the partition key, only the first\n    explicitly defined field is available for this option.\n  * `:auto_increment` - when `true` and this field is non-partitioned key, Tablestore\n    automatically generates the primary key value, which is unique in the partition key, and which\n    increases progressively, when `true` and this field is a partition key, use `ex_aliyun_ots`'s\n    Sequence to build a serial number for this field, the `auto_increment: true` option only\n    allows binding of one primary key.","ref":"EctoTablestore.Migration.html#add/3-options","title":"Options - EctoTablestore.Migration.add/3","type":"macro"},{"doc":"Adds a column what already be pre-defined column when creating a secondary index.","ref":"EctoTablestore.Migration.html#add_column/1","title":"EctoTablestore.Migration.add_column/1","type":"macro"},{"doc":"Adds a pre-defined column when creating a table.\n\nThis function only accepts types as `:integer` | `:double` | `:boolean` | `:string` | `:binary`.\n\nFor more information see the  [Chinese Docs](https://help.aliyun.com/document_detail/91947.html) | [English Docs](https://www.alibabacloud.com/help/doc-detail/91947.html)","ref":"EctoTablestore.Migration.html#add_column/2","title":"EctoTablestore.Migration.add_column/2","type":"macro"},{"doc":"create table(\"posts\") do\n      add_pk(:id, :integer, partition_key: true)\n      add_pk(:owner_id, :string)\n      add_column(:title, :string)\n      add_column(:content, :string)\n    end","ref":"EctoTablestore.Migration.html#add_column/2-examples","title":"Examples - EctoTablestore.Migration.add_column/2","type":"macro"},{"doc":"Adds a secondary index when creating a table.\n\nFor more information see the [Chinese Docs](https://help.aliyun.com/document_detail/91947.html) | [English Docs](https://www.alibabacloud.com/help/doc-detail/91947.html)","ref":"EctoTablestore.Migration.html#add_index/4","title":"EctoTablestore.Migration.add_index/4","type":"macro"},{"doc":"create table(\"posts\") do\n      add_pk(:id, :integer, partition_key: true)\n      add_pk(:owner_id, :string)\n      add_column(:title, :string)\n      add_column(:content, :string)\n      add_index(\"posts_owner\", [:owner_id, :id], [:title, :content])\n      add_index(\"posts_title\", [:title, :id], [:content], type: :global)\n    end","ref":"EctoTablestore.Migration.html#add_index/4-examples","title":"Examples - EctoTablestore.Migration.add_index/4","type":"macro"},{"doc":"* `:index_type`, the type of the index, optional. Valid values: `:global` and `:local`. By default it is `:global`.\n    - If `:index_type` is not specified or is set to `:global`, the global secondary index feature is used.\n    If you use the global secondary index feature, Tablestore automatically synchronizes the columns to be indexed and data in primary key columns from a data table to an index table in asynchronous mode.\n    The synchronization latency is within a few milliseconds.\n    - If `:index_type` is set to `:local`, the local secondary index feature is used.\n    If you use the local secondary index feature, Tablestore automatically synchronizes data from the indexed columns and the primary key columns of a data table to the columns of an index table in synchronous mode.\n    After the data is written to the data table, you can query the data from the index table.","ref":"EctoTablestore.Migration.html#add_index/4-options","title":"Options - EctoTablestore.Migration.add_index/4","type":"macro"},{"doc":"Adds a primary key when creating a secondary index.","ref":"EctoTablestore.Migration.html#add_pk/1","title":"EctoTablestore.Migration.add_pk/1","type":"macro"},{"doc":"Adds a primary key when creating a table.\n\nSame as `add/2`, see `add/2` for more information.","ref":"EctoTablestore.Migration.html#add_pk/3","title":"EctoTablestore.Migration.add_pk/3","type":"macro"},{"doc":"Define the primary key(s) of the table to create.\n\nBy default, the table will also include an `:id` primary key field (it is also partition key)\nthat has a type of `:integer` which is an autoincrementing column. Check the `table/2` docs for\nmore information.\n\nThere are up to 4 primary key(s) can be added when creation.","ref":"EctoTablestore.Migration.html#create/2","title":"EctoTablestore.Migration.create/2","type":"macro"},{"doc":"create table(\"posts\") do\n      add :title, :string\n    end\n\n    # The above is equivalent to\n\n    create table(\"posts\") do\n      add :id, :integer, partition_key: true, auto_increment: true\n      add :title, :string\n    end","ref":"EctoTablestore.Migration.html#create/2-example","title":"Example - EctoTablestore.Migration.create/2","type":"macro"},{"doc":"Drops one of the following:\n\n  * a table\n  * a secondary index\n  * a search index","ref":"EctoTablestore.Migration.html#drop/1","title":"EctoTablestore.Migration.drop/1","type":"function"},{"doc":"drop table(\"posts\")\n    drop secondary_index(\"posts\", \"posts_owner\")\n    drop search_index(\"posts\", \"posts_index\")","ref":"EctoTablestore.Migration.html#drop/1-examples","title":"Examples - EctoTablestore.Migration.drop/1","type":"function"},{"doc":"","ref":"EctoTablestore.Migration.html#drop/2","title":"EctoTablestore.Migration.drop/2","type":"function"},{"doc":"Drops a table or index if it exists.\n\nDoes not raise an error if the specified table or index does not exist.","ref":"EctoTablestore.Migration.html#drop_if_exists/1","title":"EctoTablestore.Migration.drop_if_exists/1","type":"function"},{"doc":"drop_if_exists table(\"posts\")\n    drop_if_exists secondary_index(\"posts\", \"posts_owner\")\n    drop_if_exists search_index(\"posts\", \"posts_index\")","ref":"EctoTablestore.Migration.html#drop_if_exists/1-examples","title":"Examples - EctoTablestore.Migration.drop_if_exists/1","type":"function"},{"doc":"Returns a search index struct that can be given to `create/2`.\n\nFor more information see the  [Chinese Docs](https://help.aliyun.com/document_detail/117452.html) | [English Docs](https://www.alibabacloud.com/help/doc-detail/117452.html)","ref":"EctoTablestore.Migration.html#search_index/3","title":"EctoTablestore.Migration.search_index/3","type":"function"},{"doc":"create search_index(\"posts\", \"posts_owner\") do\n      field_schema_keyword(\"title\")\n      field_schema_keyword(\"content\")\n      field_sort(\"title\")\n    end\n\nPlease refer the [link](https://hexdocs.pm/ex_aliyun_ots/ExAliyunOts.Search.html#define-field-schema) to the available field schema definition options,\nand the [link](https://hexdocs.pm/ex_aliyun_ots/ExAliyunOts.Search.html#sort) to the available sort options.","ref":"EctoTablestore.Migration.html#search_index/3-examples","title":"Examples - EctoTablestore.Migration.search_index/3","type":"function"},{"doc":"Returns a secondary index struct that can be given to `create/2`.\n\nFor more information see the  [Chinese Docs](https://help.aliyun.com/document_detail/91947.html) | [English Docs](https://www.alibabacloud.com/help/doc-detail/91947.html)","ref":"EctoTablestore.Migration.html#secondary_index/3","title":"EctoTablestore.Migration.secondary_index/3","type":"function"},{"doc":"create secondary_index(\"posts\", \"posts_owner\") do\n      add_pk(:owner_id)\n      add_pk(:id)\n      add_column(:title)\n      add_column(:content)\n    end\n\n    create secondary_index(\"posts\", \"posts_title\", index_type: :global) do\n      add_pk(:title)\n      add_pk(:id)\n      add_column(:content)\n    end","ref":"EctoTablestore.Migration.html#secondary_index/3-examples","title":"Examples - EctoTablestore.Migration.secondary_index/3","type":"function"},{"doc":"* `:include_base_data`, specifies whether the index table includes the existing data in the base table, if set it to\n  `true` means the index includes the existing data, if set it to `false` means the index excludes the existing data,\n  optional, by default it is `true`.\n\n  * `:index_type`, the type of the index, optional. Valid values: `:global` and `:local`. By default it is `:global`.\n    - If `:index_type` is not specified or is set to `:global`, the global secondary index feature is used.\n    If you use the global secondary index feature, Tablestore automatically synchronizes the columns to be indexed and data in primary key columns from a data table to an index table in asynchronous mode.\n    The synchronization latency is within a few milliseconds.\n    - If `:index_type` is set to `:local`, the local secondary index feature is used.\n    If you use the local secondary index feature, Tablestore automatically synchronizes data from the indexed columns and the primary key columns of a data table to the columns of an index table in synchronous mode.\n    After the data is written to the data table, you can query the data from the index table.","ref":"EctoTablestore.Migration.html#secondary_index/3-options","title":"Options - EctoTablestore.Migration.secondary_index/3","type":"function"},{"doc":"Returns a table struct that can be given to `create/2`.\n\nSince Tablestore is a NoSQL service, there are up to 4 primary key(s) can be added when\ncreation, the first added key is partition key when set `partition_key` option as false.","ref":"EctoTablestore.Migration.html#table/2","title":"EctoTablestore.Migration.table/2","type":"function"},{"doc":"create table(\"products\") do\n      add :name, :string\n      add :price, :integer\n    end\n\n    create table(\"products\", partition_key: false) do\n      add :name, :string\n      add :price, :integer\n    end","ref":"EctoTablestore.Migration.html#table/2-examples","title":"Examples - EctoTablestore.Migration.table/2","type":"function"},{"doc":"* `:partition_key` - as `true` by default, and there will add an `:id` field as partition key\n    with type as a large autoincrementing integer (as `bigserial`), Tablestore does not support\n    `bigserial` type for primary keys, but can use the `ex_aliyun_ots` lib's wrapper - Sequence to\n    implement it; when `false`, a partition key field is not generated on table creation.\n  * `:prefix` - the prefix for the table.\n  * `:meta` - define the meta information when create table, can see Tablestore's document for details:\n\n    * `:reserved_throughput_write` - reserve the throughput for write when create table, an\n      integer, the default value is 0;\n    * `:reserved_throughput_read` - reserve the throughput for read when create table, an integer,\n      the default value is 0;\n    * `:time_to_live` - the survival time of the saved data, a.k.a TTL; an integer, unit as second,\n      the default value is -1 (permanent preservation);\n    * `:deviation_cell_version_in_sec` - maximum version deviation, the default value is 86400\n      seconds, which is 1 day;\n    * `stream_spec` - set the stream specification of Tablestore:\n      - `is_enabled`, open or close stream\n      - `expiration_time`, the expiration time of the table's stream\n    * `enable_local_txn` - specifies whether to enable the local transaction feature.\n      The value of this parameter is of the `:boolean` type. Default value: false.\n      If you want to enable the local transaction feature when you create a data table, set this parameter to true.","ref":"EctoTablestore.Migration.html#table/2-options","title":"Options - EctoTablestore.Migration.table/2","type":"function"},{"doc":"# Alibaba Tablestore adapter for Ecto\n\n[![Module Version](https://img.shields.io/hexpm/v/ecto_tablestore.svg)](https://hex.pm/packages/ecto_tablestore)\n[![Hex Docs](https://img.shields.io/badge/hex-docs-lightgreen.svg)](https://hexdocs.pm/ecto_tablestore/)\n[![Total Download](https://img.shields.io/hexpm/dt/ecto_tablestore.svg)](https://hex.pm/packages/ecto_tablestore)\n[![License](https://img.shields.io/hexpm/l/ecto_tablestore.svg)](https://github.com/edragonconnect/ecto_tablestore/blob/master/LICENSE.md)\n[![Last Updated](https://img.shields.io/github/last-commit/edragonconnect/ecto_tablestore.svg)](https://github.com/edragonconnect/ecto_tablestore/commits/master)","ref":"readme.html","title":"Alibaba Tablestore adapter for Ecto","type":"extras"},{"doc":"If you have used this library and upgrade it into `0.11.0` or later, there are some breaking changes in `0.11.0`,\nyou may need to make a data migration before upgrade and one use case check, please see this\n[changelog](https://github.com/edragonconnect/ecto_tablestore/blob/master/CHANGELOG.md)\nfor details, if you are new to use `0.11.0` or later, please ignore this comment.","ref":"readme.html#notice","title":"Notice - Alibaba Tablestore adapter for Ecto","type":"extras"},{"doc":"Ecto 3.x adapter for [Alibaba Tablestore](https://www.alibabacloud.com/product/table-store), this\nis built on top of [`ex_aliyun_ots`](https://hex.pm/packages/ex_aliyun_ots) to implement\n`Ecto.Adapter` and `Ecto.Adapter.Schema` behaviours.\n\nSupported features:\n\n* Compatible `Ecto.Repo` API.\n\n* Support schema's `timestamps()` macro, make `inserted_at` and `updated_at` as an integer UTC\n  timestamps.\n\n* Support `:map` | `{:map, _}` | `:array` | `{:array, _}` field type, use Jason to encode the\n  field value into :string when save, use Jason to simply decode the field value into `:map` |\n  `:array` when read, the :keys option of Jason's decode always use :string.\n\n* Support the partition key is autoincrementing, use the sequence feature provided by\n  `ex_aliyun_ots`.\n\n* Automatically converts the returned original row results into corresponding schema(s).\n\n* Automatically generate the provided attribute-column field(s) of schema entity into the `filter`\n  expression option of `GetRow` (see `c:EctoTablestore.Repo.one/2`) and `BatchGet` (see\n  `c:EctoTablestore.Repo.batch_get/1`) when use `entity_full_match: true`, by default this option is\n  `false`.\n\n* Automatically generate the provided attribute-column field(s) of schema entity into the\n  `condition` expression option of `BatchWrite` (see `c:EctoTablestore.Repo.batch_write/2`) when\n  use `entity_full_match: true`, by default this option is `false`.\n\n* Automatically map changeset's attribute-column field(s) into `UpdateRow` operation when call\n  `c:EctoTablestore.Repo.update/2`:\n\n  * Use atomic increment via `{:increment, integer()}` in changeset, and return the increased\n    value in the corresponding field(s) by default;\n  * Set any attribute(s) of schema changeset as `nil` will `:delete_all` that attribute-column\n    field(s);\n  * Set existed attribute(s) in schema changeset will `:put` to save.\n\n* Support embedded schema, please refer full functions of `Ecto.Schema` for details.\n\n* Automatically use schema defined attribute fields into `:columns_to_get` in all read operations\n  by default if this option is not explicitly provided.\n\nImplement Tablestore row related functions in `EctoTablestore.Repo` module, please see\n[document](https://hexdocs.pm/ecto_tablestore/readme.html) for details:\n\n* PutRow\n* GetRow\n* UpdateRow\n* DeleteRow\n* GetRange\n* StreamRange\n* BatchGetRow\n* BatchWriteRow\n* Search","ref":"readme.html#introduce","title":"Introduce - Alibaba Tablestore adapter for Ecto","type":"extras"},{"doc":"Provide a simple migration to create or drop the table, the secondary index and the search index, please see\n`EctoTablestore.Migration` for details.","ref":"readme.html#migration","title":"Migration - Alibaba Tablestore adapter for Ecto","type":"extras"},{"doc":"1, Configure `My` instance(s) information of Alibaba Tablestore product.\n\n```elixir\nuse Mix.Config\n\n# config for `ex_aliyun_ots`\n\nconfig :ex_aliyun_ots, MyInstance,\n  name: \"MY_INSTANCE_NAME\",\n  endpoint: \"MY_INSTANCE_ENDPOINT\",\n  access_key_id: \"MY_OTS_ACCESS_KEY\",\n  access_key_secret: \"MY_OTS_ACCESS_KEY_SECRET\"\n\nconfig :ex_aliyun_ots,\n  instances: [MyInstance]\n\n# config for `ecto_tablestore`\n\nconfig :my_otp_app, EctoTablestore.MyRepo,\n  instance: MyInstance\n```\n\n2, Create the `EctoTablestore.MyRepo` module mentioned earlier in the configuration, use\n`EctoTablestore.Repo` and set required `otp_app` option with your OTP application's name.\n\n```elixir\ndefmodule EctoTablestore.MyRepo do\n  use EctoTablestore.Repo,\n    otp_app: :my_otp_app\nend\n```\n\n3, Each repository in Ecto defines a `start_link/0` function that needs to be invoked before using\nthe repository. In general, this function is not called directly, but used as part of your\napplication supervision tree.\n\nAdd `EctoTablestore.MyRepo` into your application start callback that defines and start your\nsupervisor. You just need to edit `start/2` function to start the repo as a supervisor on your\napplication's supervisor:\n\n```elixir\ndef start(_type, _args) do\n  children = [\n    {EctoTablestore.MyRepo, []}\n  ]\n\n  opts = [strategy: :one_for_one, name: MyApp.Supervisor]\n  Supervisor.start_link(children, opts)\nend\n```\n\n4, After finish the above preparation, we can use `MyRepo` to operate data store.","ref":"readme.html#usage","title":"Usage - Alibaba Tablestore adapter for Ecto","type":"extras"},{"doc":"Base on unique integer of atomic-increment sequence, provides a way to simply integrate `Hashids`\nto generate your *hash* ids as the primary key when insert row(s), for `Repo.insert/2` or `Repo.batch_write/1`.\n\nAdd a dependency to your Mix project\n\n```elixir\n{:hashids, \"~> 2.0\"}\n```\n\n#","ref":"readme.html#integrate-hashids","title":"Integrate Hashids - Alibaba Tablestore adapter for Ecto","type":"extras"},{"doc":"```elixir\ndefmodule Module do\n  use EctoTablestore.Schema\n\n  schema \"table_name\" do\n    field(:id, Ecto.Hashids, primary_key: true, autogenerate: true,\n      hashids: [salt: \"123\", min_len: 2, alphabet: \"...\"])\n    field(:content, :string)\n  end\n\nend\n```\n\nOptions:\n\n* The `primary_key` as true is required;\n* The `autogenerate` as true is required;\n* The `salt`, `min_len` and `alphabet` of hashids options are used for configuration options from\n  `Hashids.new/1`.\n\nNotice: the `salt`, `min_len` and `alphabet` of hashids options also can be configured in the `:ecto_tablestore`\nconfig for each defined schema, for example:\n\n```elixir\nconfig :ecto_tablestore,\n  hashids: [\n    {Module, salt: \"...\", min_len: 16, alphabet: \"...\"},\n    ...\n  ]\n```\n\n#","ref":"readme.html#use-in-schema","title":"Use in schema - Alibaba Tablestore adapter for Ecto","type":"extras"},{"doc":"Use `:hashids` as a type in `add` operation to define the partition key, the principle behind this\nwill use `ecto_tablestore_default_seq` as a global default table to maintain the sequences of all\ntables, if `ecto_tablestore_default_seq` is not existed, there will create this, if it is existed,\nplease ignore the \"OTSObjectAlreadyExist\" error of requested table already exists.\n\n```elixir\ndefmodule EctoTablestore.TestRepo.Migrations.TestHashids do\n  use EctoTablestore.Migration\n\n  def change do\n    create table(\"table_name\") do\n      add :id, :hashids, partition_key: true\n      add :oid, :integer\n    end\n  end\n\nend\n```\n\nOptions:\n\n* The `partition_key` option as true is required;\n* The `auto_increment` option is ignored when the partition key is `:hashids` type.","ref":"readme.html#use-in-migration","title":"Use in migration - Alibaba Tablestore adapter for Ecto","type":"extras"},{"doc":"Alibaba Tablestore product official references:\n\n* [English document](https://www.alibabacloud.com/help/doc-detail/27280.htm)\n* [中文文档](https://help.aliyun.com/document_detail/27280.html)","ref":"readme.html#references","title":"References - Alibaba Tablestore adapter for Ecto","type":"extras"},{"doc":"This project is licensed under the MIT license. Copyright (c) 2018- Xin Zou.","ref":"readme.html#license","title":"License - Alibaba Tablestore adapter for Ecto","type":"extras"},{"doc":"# Changelog","ref":"changelog.html","title":"Changelog","type":"extras"},{"doc":"* Add feature for local secondary index creation in migration ([#44](https://github.com/edragonconnect/ecto_tablestore/pull/44)).\n* Fix case match to `c:EctoTablestore.Repo.transaction/2` ([#45](https://github.com/edragonconnect/ecto_tablestore/pull/45), [#46](https://github.com/edragonconnect/ecto_tablestore/pull/46)).\n* Fix document source link was broken when entry from hexdocs.pm ([#47](https://github.com/edragonconnect/ecto_tablestore/pull/47)).\n* Make macro definition expandable in migration ([#48](https://github.com/edragonconnect/ecto_tablestore/pull/48)).","ref":"changelog.html#v0-15-1-2024-03-18","title":"v0.15.1 (2024-03-18) - Changelog","type":"extras"},{"doc":"* Add `c:EctoTablestore.Repo.sql_query/2` ([#40](https://github.com/edragonconnect/ecto_tablestore/pull/40)).\n* Add `c:EctoTablestore.Repo.transaction/2` and `c:EctoTablestore.Repo.rollback/1` base on the local transaction ([#41](https://github.com/edragonconnect/ecto_tablestore/pull/41), [#42](https://github.com/edragonconnect/ecto_tablestore/pull/42)).","ref":"changelog.html#v0-15-0-2024-01-26","title":"v0.15.0 (2024-01-26) - Changelog","type":"extras"},{"doc":"* Add `Repo.one!/2`.\n* Enhance `Repo.get_range/2`, `Repo.stream/2` with `token` option.\n* Enhance output when create search index.","ref":"changelog.html#v0-14-0-2023-01-04","title":"v0.14.0 (2023-01-04) - Changelog","type":"extras"},{"doc":"* Fix and update document to Repo's batch get.\n* Fix to use the public API `Ecto.Adapter.lookup_meta/1` when upgrade to `ecto ~> 3.8.*`.","ref":"changelog.html#v0-13-3-2022-05-24","title":"v0.13.3 (2022-05-24) - Changelog","type":"extras"},{"doc":"* Fix schema's timestamps() as as the original with second time unit.\n* Fix `Repo.get_range/4` document missed.\n\n## ~~v0.13.1 (2021-10-25)~~\n\n* Update document to clarify `EctoTablestore.Schema` use.\n* Some changes to schema's timestamps() but with unexpected change the time unit of `inserted_at`/`updated_at` fields from second to nanosecond, fixed in `0.13.2`, so please DO NOT use this version.","ref":"changelog.html#v0-13-2-2022-01-21","title":"v0.13.2 (2022-01-21) - Changelog","type":"extras"},{"doc":"* Refactor `:hashids` type with Ecto.ParameterizedType.\n* Clean logger when create sequence.\n* Fix warning: function checked_out?/1 required with ecto `3.7.0`.","ref":"changelog.html#v0-13-0-2021-08-25","title":"v0.13.0 (2021-08-25) - Changelog","type":"extras"},{"doc":"* Some fix and enhancement via [#34](https://github.com/edragonconnect/ecto_tablestore/pull/34):\n\n  1. Fix to properly adapt Ecto type on load;\n  2. Add support to `:decimal` type in the field of schema;\n  3. Add `Ecto.ReplaceableString` type.","ref":"changelog.html#v0-12-2-2021-08-20","title":"v0.12.2 (2021-08-20) - Changelog","type":"extras"},{"doc":"* Use schema defined attribute fields into `:columns_to_get` option as default\n  in all read operations via [#33](https://github.com/edragonconnect/ecto_tablestore/pull/33).","ref":"changelog.html#v0-12-1-2021-08-11","title":"v0.12.1 (2021-08-11) - Changelog","type":"extras"},{"doc":"* Some fix and enhancement via [#32](https://github.com/edragonconnect/ecto_tablestore/pull/32):\n\n  1. Fix fail when batch write with `embeds_many` or `embeds_one` fields;\n  2. Add `:entity_full_match` as an optional option to leverage the provided attribute columns of the struct into the filter of the batch write\n     condition when it is `true`, default value is `false`. After this breaking change, each put/update/delete operation of batch write requires\n     explicitly set `:condition` option, excepts the following `#3` case;\n  3. When insert or a `:put` operation of batch write with a table defined an auto increment primary key(a server side function), we can omit\n     `:condition` option, because the server side only accepts `condition: condition(:ignore)`, we internally wrapper this to simplify the\n     input in this use case;\n  4. Clarify document about the `:condition` option of insert/update/delete/batch_write;\n  5. Some code naming improvement.","ref":"changelog.html#v0-12-0-2021-08-10","title":"v0.12.0 (2021-08-10) - Changelog","type":"extras"},{"doc":"* Add `drop_if_exists/1` function for migration.\n* Add `--migrations_path` option to `mix ecto.ots.migrate`.","ref":"changelog.html#v0-11-2-2021-05-21","title":"v0.11.2 (2021-05-21) - Changelog","type":"extras"},{"doc":"* Fix format error to put the `.formatter.exs` file into package.","ref":"changelog.html#v0-11-1-2021-05-17","title":"v0.11.1 (2021-05-17) - Changelog","type":"extras"},{"doc":"* [Need to migrate before upgrade] When define the type of partition key as an auto increment integer, we\n  created a `\"#{table_name}_seq\"` table as an internal sequence to the `\"#{table_name}\"` table,\n  but now all sequence(s) managed into the `\"ecto_tablestore_default_seq\"` table, see\n  [#16](https://github.com/edragonconnect/ecto_tablestore/pull/16).\n* Add `Repo.stream_search/3`, see [#17](https://github.com/edragonconnect/ecto_tablestore/pull/16).\n* Add `Repo.stream/2`, see [#21](https://github.com/edragonconnect/ecto_tablestore/pull/21).\n* Add `Repo.get_range/1` and `Repo.get_range/2` for easy use, see\n  [#28](https://github.com/edragonconnect/ecto_tablestore/pull/28).\n* Support changeset optimistic_lock in update, see commit\n  [66f6823](https://github.com/edragonconnect/ecto_tablestore/commit/66f6823704f14940e97f8195e63ad2c29b77ecea).\n* [Need to check before upgrade] Fix update with `:increment` operation return an unexpected result when\n  schema has multi integer fields defined, and require to explicitly add column name(s) which are related in the atomic increment\n  operation into the `:returning` option of `Repo.update/2`,\n  see [#19](https://github.com/edragonconnect/ecto_tablestore/pull/19).\n* Fix to properly handle the stale error case when insert,\n  see [#20](https://github.com/edragonconnect/ecto_tablestore/pull/20).\n* Fix to remove the useless supervisor name to resolve duplicated naming conflict in some cases,\n  see [#24](https://github.com/edragonconnect/ecto_tablestore/pull/24).","ref":"changelog.html#v0-11-0-2021-05-14","title":"v0.11.0 (2021-05-14) - Changelog","type":"extras"}]}